@@ -18,8 +18,6 @@
 #include <sys/time.h>
 /*#include <sys/utsname.h>*/
 
-#define DEBUGFILE       "/tmp/abstractile.dbg"
-
 #define MODE_CREATE		0   /* init, create, then finish sleep */
 #define MODE_ERASE		1   /* erase, then reset colors */
 #define MODE_DRAW		2
@@ -92,7 +90,8 @@ struct state {
     /* draw, erase, fill, init, line, object, z indexes */
   unsigned int di, ei, fi, ii, bi, li, eli, oi, zi;
   /* size variables */
-  unsigned int gridx, gridy, gridn;                     /* grid size */
+  int gridx, gridy;                     /* grid size */
+  unsigned int gridn;
   int lwid, bwid, swid;/* line width, background width, shadow width */
   int narray, max_wxh;
   int elwid, elpu, egridx, egridy; /* for now */
@@ -207,6 +206,7 @@ _init_zlist(struct state *st)
 
   st->gridx=st->xgwa.width/st->lwid;
   st->gridy=st->xgwa.height/st->lwid;
+  if ((st->gridx <= 0) || (st->gridy <= 0)) abort();
   st->gridn=st->gridx*st->gridy;
   /* clear grid */
   for (z=0; z<st->gridn; z++) {
@@ -663,6 +663,9 @@ _init_screen(struct state *st)
   if (st->tile==TILE_THIN)
     st->lwid=2;
 
+  if (st->xgwa.width > 2560 || st->xgwa.height > 2560)
+    st->lwid *= 3;  /* Retina displays */
+
   _init_zlist(st);
 
   st->maxlen=(st->lwid>6) ? 2+(random()%4) :
@@ -916,6 +919,7 @@ _getcolor(struct state *st, int x, int y)
 {
   int n, cv[LAYERS];
 
+  cv[0] = 0;
   for (n=0; n<st->layers; n++) {
     cv[n]=_pattern(st,x,y,n);
                   /* first wave/shape */
@@ -1079,8 +1083,13 @@ _newline(struct state *st)
     True : False;
   st->dline[st->li].obj = (lt==LINE_NEW) ? st->oi :
     st->dline[bl].obj;
-  st->dline[st->li].color = (lt==LINE_NEW) ?
-    (_getcolor(st,x,y))%st->ncolors : st->dline[bl].color;
+  if (lt==LINE_NEW) {
+    int color =  (_getcolor(st,x,y))%st->ncolors;
+    if (color < 0) color += st->ncolors;
+    st->dline[st->li].color = color;
+  } else {
+    st->dline[st->li].color = st->dline[bl].color;
+  }
   st->dline[st->li].deo=(_getdeo(st,x,y,st->dmap,1) +
     (random()%st->dvar) + (random()%st->dvar))*st->ddir;
   st->dline[st->li].ndol=0;
@@ -1479,6 +1488,7 @@ abstractile_init(Display *display, Window window)
       fprintf(stderr, "%s: unknown tile option %s\n", progname, tile);
     st->tile = TILE_RANDOM;
   }
+  if (tile) free (tile);
 
   st->speed = get_integer_resource(display, "speed", "Integer");
   if (st->speed < 0) st->speed = 0;
@@ -1559,6 +1569,13 @@ abstractile_reshape (Display *dpy, Window window, void *closure,
 static Bool
 abstractile_event (Display *dpy, Window window, void *closure, XEvent *event)
 {
+  struct state *st = (struct state *) closure;
+  if (screenhack_event_helper (dpy, window, event))
+    {
+      st->mode=MODE_CREATE;
+      return True;
+    }
+
   return False;
 }
 
@@ -1566,6 +1583,14 @@ static void
 abstractile_free (Display *dpy, Window window, void *closure)
 {
   struct state *st = (struct state *) closure;
+  if (st->dline) free(st->dline);
+  if (st->eline) free(st->eline);
+  if (st->grid)  free(st->grid);
+  if (st->zlist) free(st->zlist);
+  if (st->fdol)  free(st->fdol);
+  if (st->odi)   free(st->odi);
+  XFreeGC (dpy, st->fgc);
+  XFreeGC (dpy, st->bgc);
   free (st);
 }
 
@@ -1576,6 +1601,9 @@ static const char *abstractile_defaults [] = {
   "*sleep:             3",
   "*speed:             3",
   "*tile:         random",
+#ifdef HAVE_MOBILE
+  "*ignoreRotation: True",
+#endif
   0
 };
 
