@@ -1,4 +1,4 @@
-/* xanalogtv-cli, Copyright (c) 2018-2020 Jamie Zawinski <jwz@jwz.org>
+/* xanalogtv-cli, Copyright © 2018-2022 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -36,6 +36,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 
+#include "screenhackI.h"
 #include "resources.h"
 #include "visual.h"
 #include "yarandom.h"
@@ -54,10 +55,6 @@ const char *progclass;
 int mono_p = 0;
 static Bool verbose_p = 0;
 
-#define RANDSIGN() ((random() & 1) ? 1 : -1)
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 #define MAX_MULTICHAN 2
 static int N_CHANNELS=12;
 static int MAX_STATIONS=6;
@@ -120,6 +117,9 @@ XCreateGC(Display *dpy, Drawable d, unsigned long mask, XGCValues *gcv)
   return 0;
 }
 
+int screen_number (Screen *screen) { return 0; }
+
+
 XImage *
 XCreateImage (Display *dpy, Visual *v, unsigned int depth,
               int format, int offset, char *data,
@@ -345,12 +345,6 @@ get_pixel_resource (Display *dpy, Colormap cmap, char *name, char *class)
   abort();
 }
 
-XFontStruct *
-load_font_retry (Display *dpy, const char *xlfd)
-{
-  abort();
-}
-
 Bool
 put_xshm_image (Display *dpy, Drawable d, GC gc, XImage *image,
                 int src_x, int src_y, int dest_x, int dest_y,
@@ -451,8 +445,9 @@ update_smpte_colorbars(analogtv_input *input)
     {
       double aspect = (double)
         st->output_frame->width / st->output_frame->height;
-      int w2 = st->tv->xgwa.width  * 0.35;
-      int h2 = st->tv->xgwa.height * 0.35 * aspect;
+      double scale = (aspect > 1 ? 0.35 : 0.6);
+      int w2 = st->tv->xgwa.width  * scale;
+      int h2 = st->tv->xgwa.height * scale * aspect;
       analogtv_load_ximage (st->tv, input, st->logo, st->logo_mask,
                             (st->tv->xgwa.width - w2) / 2,
                             st->tv->xgwa.height * 0.20,
@@ -522,7 +517,7 @@ analogtv_save_frame (struct state *st, const char *outfile,
 # endif /* ! HAVE_LIBPNG */
 
   fclose (f);
-  if (verbose_p > 1)
+  if (verbose_p > 2)
     fprintf (stderr, "%s: wrote %s\n", progname, pngfile);
   free (pngfile);
 }
@@ -537,7 +532,7 @@ delete_tmp_files(void)
   for (i = 0; i <= st->frames_written; i++)
     {
       sprintf (outfile, st->framefile_fmt, i);
-      if (verbose_p > 2)
+      if (verbose_p > 3)
         fprintf (stderr, "%s: rm %s\n", progname, outfile);
       unlink (outfile);
     }
@@ -932,7 +927,18 @@ analogtv_convert (const char **infiles, const char *outfile,
         st->curinputi = 0;
       } else {
         /* Otherwise random */
+        int prev = st->curinputi;
+      AGAIN:
         st->curinputi = 1 + (random() % (N_CHANNELS - 1));
+
+        /* In single mode, always alternate to the unadulterated image:
+           no two noisy images in a row, always intersperse clean. */
+        if (singlep && prev != 0)
+          st->curinputi = 0;
+
+        /* In single mode, do colorbars-only a bit less often. */
+        if (singlep && st->curinputi == 1 && !(random() % 3))
+          goto AGAIN;
       }
 
       stats[st->curinputi]++;
@@ -941,7 +947,7 @@ analogtv_convert (const char **infiles, const char *outfile,
       st->tv->channel_change_cycles=200000;
 
       if (verbose_p > 1)
-        fprintf (stderr, "%s: %.1f: channel %d\n",
+        fprintf (stderr, "%s: %5.1f sec: channel %d\n",
                  progname, curticks/1000.0, st->curinputi);
 
       /* Turn the knobs every now and then */
