@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# Copyright © 2008-2018 Jamie Zawinski <jwz@jwz.org>
+# Copyright © 2008-2022 Jamie Zawinski <jwz@jwz.org>
 #
 # Permission to use, copy, modify, distribute, and sell this software and its
 # documentation for any purpose is hereby granted without fee, provided that
@@ -21,7 +21,7 @@ use diagnostics;
 use strict;
 
 my $progname = $0; $progname =~ s@.*/@@g;
-my ($version) = ('$Revision: 1.29 $' =~ m/\s(\d[.\d]+)\s/s);
+my ($version) = ('$Revision: 1.40 $' =~ m/\s(\d[.\d]+)\s/s);
 
 my $verbose = 0;
 my $debug_p = 0;
@@ -30,29 +30,29 @@ my $debug_p = 0;
 my $text_default_opts = '';
 foreach (qw(text-mode text-literal text-file text-url text-program)) {
   my $s = $_; $s =~ s/-(.)/\U$1/g; $s =~ s/url/URL/si;
-  $text_default_opts .= "{\"-$_\", \".$s\", XrmoptionSepArg, 0},\n";
+  $text_default_opts .= "{\"--$_\", \".$s\", XrmoptionSepArg, 0},\n";
 }
 my $image_default_opts = '';
 foreach (qw(choose-random-images grab-desktop-images)) {
   my $s = $_; $s =~ s/-(.)/\U$1/g;
-  $image_default_opts .= "{\"-$_\", \".$s\", XrmoptionSepArg, 0},\n";
+  $image_default_opts .= "{\"--$_\", \".$s\", XrmoptionSepArg, 0},\n";
 }
 my $xlockmore_default_opts = '';
 foreach (qw(count cycles delay ncolors size font)) {
-  $xlockmore_default_opts .= "{\"-$_\", \".$_\", XrmoptionSepArg, 0},\n";
+  $xlockmore_default_opts .= "{\"--$_\", \".$_\", XrmoptionSepArg, 0},\n";
 }
 $xlockmore_default_opts .= 
- "{\"-wireframe\", \".wireframe\", XrmoptionNoArg, \"true\"},\n" .
- "{\"-3d\", \".use3d\", XrmoptionNoArg, \"true\"},\n" .
- "{\"-no-3d\", \".use3d\", XrmoptionNoArg, \"false\"},\n";
+ "{\"--wireframe\", \".wireframe\", XrmoptionNoArg, \"true\"},\n" .
+ "{\"--3d\", \".use3d\", XrmoptionNoArg, \"true\"},\n" .
+ "{\"--no-3d\", \".use3d\", XrmoptionNoArg, \"false\"},\n";
 
 my $thread_default_opts = 
-  "{\"-threads\",    \".useThreads\", XrmoptionNoArg, \"True\"},\n" .
-  "{\"-no-threads\", \".useThreads\", XrmoptionNoArg, \"False\"},\n";
+  "{\"--threads\",    \".useThreads\", XrmoptionNoArg, \"True\"},\n" .
+  "{\"--no-threads\", \".useThreads\", XrmoptionNoArg, \"False\"},\n";
 
 my $analogtv_default_opts = '';
 foreach (qw(color tint brightness contrast)) {
-  $analogtv_default_opts .= "{\"-tv-$_\", \".TV$_\", XrmoptionSepArg, 0},\n";
+  $analogtv_default_opts .= "{\"--tv-$_\", \".TV$_\", XrmoptionSepArg, 0},\n";
 }
 
 $analogtv_default_opts .= $thread_default_opts;
@@ -61,7 +61,7 @@ $analogtv_default_opts .= $thread_default_opts;
 
 # Returns two tables:
 # - A table of the default resource values.
-# - A table of "-switch" => "resource: value", or "-switch" => "resource: %"
+# - A table of "--switch" => "resource: value", or "--switch" => "resource: %"
 #
 sub parse_src($) {
   my ($saver) = @_;
@@ -73,14 +73,13 @@ sub parse_src($) {
   $file = 'b_lockglue.c' if ($file eq 'bubble3d.c');
   $file = 'polyhedra-gl.c' if ($file eq 'polyhedra.c');
   $file = 'companion.c' if ($file eq 'companioncube.c');
-  $file = 'rd-bomb.c' if ($file eq 'rdbomb.c');
 
   my $ofile = $file;
   $file = "glx/$ofile"          unless (-f $file);
   $file = "../hacks/$ofile"     unless (-f $file);
   $file = "../hacks/glx/$ofile" unless (-f $file);
   my $body = '';
-  open (my $in, '<', $file) || error ("$ofile: $!");
+  open (my $in, '<:utf8', $file) || error ("$ofile: $!");
   while (<$in>) { $body .= $_; }
   close $in;
   $file =~ s@^.*/@@;
@@ -103,6 +102,7 @@ sub parse_src($) {
       s/^\s*//s;
       s/\s*$//s;
       next if m/^0?$/s;
+      s/"[ \t]*\n\s*"//s;  # string-append continuation lines
       my ($key, $val) = m@^\"([^:\s]+)\s*:\s*(.*?)\s*\"$@;
       print STDERR "$progname: $file: unparsable: $_\n" unless $key;
       $key =~ s/^[.*]//s;
@@ -155,11 +155,11 @@ sub parse_src($) {
   print STDERR "$progname: $file: switches to resources:\n"
     if ($verbose > 2);
   my %switch_to_res;
-  $switch_to_res{'-fps'} = 'doFPS: true';
-  $switch_to_res{'-fg'}  = 'foreground: %';
-  $switch_to_res{'-bg'}  = 'background: %';
-  $switch_to_res{'-no-grab-desktop-images'}  = 'grabDesktopImages: false';
-  $switch_to_res{'-no-choose-random-images'}  = 'chooseRandomImages: false';
+  $switch_to_res{'--fps'} = 'doFPS: true';
+  $switch_to_res{'--fg'}  = 'foreground: %';
+  $switch_to_res{'--bg'}  = 'background: %';
+  $switch_to_res{'--no-grab-desktop-images'}  = 'grabDesktopImages: false';
+  $switch_to_res{'--no-choose-random-images'}  = 'chooseRandomImages: false';
 
   my ($ign, $opts) = ($body =~ m/(_options|\bopts)\s*\[\]\s*=\s*{(.*?)}\s*;/s);
   if  ($xlockmore_p || $thread_p || $analogtv_p || $opts) {
@@ -180,12 +180,13 @@ sub parse_src($) {
                   \s * \"([^\"]+)\" \s* ,
                   \s * ([^\s]+)     \s* ,
                   \s * (\"([^\"]*)\"|([a-zA-Z\d_]+)) \s* }@xi;
+      $switch =~ s/^(-[^-])/-$1/s;  # "-foo" => "--foo"
       print STDERR "$progname: $file: unparsable: $_\n" unless $switch;
       my $val = defined($v1) ? $v1 : $v2;
       $val = '%' if ($type eq 'XrmoptionSepArg');
       $res =~ s/^[.*]//s;
       $res =~ s/^[a-z\d]+\.//si;
-      $switch =~ s/^\+/-no-/s;
+      $switch =~ s/^\+/--no-/s;
 
       $val = "$res: $val";
       if (defined ($switch_to_res{$switch})) {
@@ -221,7 +222,7 @@ sub parse_xml($$$) {
   my $ofile = $file;
   $file = "../hacks/$ofile" unless (-f $file);
   my $body = '';
-  open (my $in, '<', $file) || error ("$ofile: $!");
+  open (my $in, '<:utf8', $file) || error ("$ofile: $!");
   while (<$in>) { $body .= $_; }
   close $in;
   $file =~ s@^.*/@@;
@@ -232,19 +233,19 @@ sub parse_xml($$$) {
     <select id="textMode">
       <option id="date"  _label="Display the date and time"/>
       <option id="text"  _label="Display static text"
-        arg-set="-text-mode literal"/>
+        arg-set="--text-mode literal"/>
       <option id="url"   _label="Display the contents of a URL"
-        arg-set="-text-mode url"/>
+        arg-set="--text-mode url"/>
     </select>
-    <string id="textLiteral" _label="Text to display" arg="-text-literal %"/>
-    <string id="textURL" _label="URL to display" arg="-text-url %"/>
+    <string id="textLiteral" _label="Text to display" arg="--text-literal %"/>
+    <string id="textURL" _label="URL to display" arg="--text-url %"/>
     @gs;
 
   $body =~ s@<xscreensaver-image\s*/?>@
     <boolean id="grabDesktopImages" _label="Grab screenshots"
-       arg-unset="-no-grab-desktop-images"/>
+       arg-unset="--no-grab-desktop-images"/>
     <boolean id="chooseRandomImages" _label="Use photo library"
-       arg-unset="-no-choose-random-images"/>
+       arg-unset="--no-choose-random-images"/>
     @gs;
 
   $body =~ s/<!--.*?-->/ /gsi;
@@ -386,6 +387,7 @@ sub parse_xml($$$) {
         my ($set)   = $ctrl->{'arg-set'};
         my ($unset) = $ctrl->{'arg-unset'};
         my ($arg) = $set || $unset || error ("$file: unparsable: $args");
+        $arg = lc($arg);
         my ($res) = $switch_to_res->{$arg};
           error ("$file: no resource for boolean switch \"$arg\"") unless $res;
 
@@ -440,11 +442,12 @@ sub parse_xml($$$) {
 }
 
 
-sub check_config($) {
-  my ($saver) = @_;
+sub check_config($$) {
+  my ($saver, $android_p) = @_;
 
   # kludge
   return 0 if ($saver =~ m/(-helper)$/);
+  return 0 if ($saver =~ m/(xscreensaver-)/);
 
   my ($src_opts, $switchmap) = parse_src ($saver);
   my ($saver_title, $gl_p, $xml_opts, $widgets) =
@@ -461,6 +464,8 @@ sub check_config($) {
         if ($verbose > 1);
     } elsif (!defined($sval)) {
       print STDERR "$progname: $saver: $res: not in source\n";
+    } elsif ($res eq 'progclass') {
+      # Ignore this one
     } elsif ($claim !~ m/ = %$/s &&
              ($compare eq '!='
               ? $sval eq $xval
@@ -477,7 +482,7 @@ sub check_config($) {
   # the XCode target name.
   #
   my $obd = "../OSX/build/Debug";
-  if (-d $obd) {
+  if (!$android_p && -d $obd) {
     my $progclass = $src_opts->{progclass};
     $progclass = 'DNAlogo' if ($progclass eq 'DNALogo');
     my $f = (glob("$obd/$progclass.saver*"))[0];
@@ -577,7 +582,7 @@ sub write_file_if_changed($$;$) {
   my ($outfile, $body, $suffix_msg) = @_;
 
   my $file_tmp = "$outfile.tmp";
-  open (my $out, '>', $file_tmp) || error ("$file_tmp: $!");
+  open (my $out, '>:utf8', $file_tmp) || error ("$file_tmp: $!");
   (print $out $body) || error ("$file_tmp: $!");
   close $out || error ("$file_tmp: $!");
   rename_or_delete ($outfile, $file_tmp, $suffix_msg);
@@ -589,7 +594,7 @@ sub write_file_if_changed($$;$) {
 sub read_template($$) {
   my ($file, $subs) = @_;
   my $body = '';
-  open (my $in, '<', $file) || error ("$file: $!");
+  open (my $in, '<:utf8', $file) || error ("$file: $!");
   while (<$in>) { $body .= $_; }
   close $in;
 
@@ -680,6 +685,8 @@ sub munge_blurb($$$$) {
                "\n" .
                $desc .
                "\n");
+  utf8::decode($desc1);   # Pack UTF-8 into wide chars.
+  utf8::decode($desc2);
 
   # unwrap lines, but only when it's obviously ok: leave blank lines,
   # and don't unwrap if that would compress leading whitespace on a line.
@@ -719,7 +726,7 @@ sub build_android(@) {
   {
     my $file = "../utils/version.h";
     my $body = '';
-    open (my $in, '<', $file) || error ("$file: $!");
+    open (my $in, '<:utf8', $file) || error ("$file: $!");
     while (<$in>) { $body .= $_; }
     close $in;
     ($vers) = ($body =~ m@ (\d+\.[0-9a-z]+) @s);
@@ -729,22 +736,21 @@ sub build_android(@) {
 
   foreach my $saver (@savers) {
     next if ($saver =~ m/(-helper)$/);
-    $saver = 'rdbomb' if ($saver eq 'rd-bomb');
 
     my ($src_opts, $switchmap) = parse_src ($saver);
     my ($saver_title, $gl_p, $xml_opts, $widgets) =
       parse_xml ($saver, $switchmap, $src_opts);
 
-    my $saver_class = "${saver_title}";
-    $saver_class =~ s/\s+//gs;
-    $saver_class =~ s/^([a-z])/\U$1/gs;  # upcase first letter
+    # The Android daydream list sorts the $saver_title strings with strcmp,
+    # meaning capitals come before lower case, and "ö" comes after "z".
+    # It doesn't matter how we order them in the XML files.
+    $saver_title =~ s/^([a-z])/\U$1/gs;  # upcase first letter
 
-    $saver_title =~ s/(.[a-z])([A-Z\d])/$1 $2/gs;	# Spaces in InterCaps
-    $saver_title =~ s/^(GL|RD)[- ]?(.)/$1 \U$2/gs;	# Space after "GL"
-    $saver_title =~ s/^Apple ?2$/Apple &#x5D;&#x5B;/gs;	# "Apple ]["
-    $saver_title =~ s/(m)oe(bius)/$1&#xF6;$2/gsi;	# &ouml;
-    $saver_title =~ s/(moir)e/$1&#xE9;/gsi;		# &eacute;
-    $saver_title =~ s/^([a-z])/\U$1/s;			# "M6502" for sorting
+    # This is a Java class name, so it could just be $saver, but it turns
+    # out that unicrud is allowed in class names, so why not.
+    my $saver_class = $saver_title;
+    $saver_class =~ s/\]\[/2/gs;
+    $saver_class =~ s/[-_\s]//gs;
 
     my $settings = '';
 
@@ -780,8 +786,7 @@ sub build_android(@) {
       my $rsrc  = $widget->{resource};
       my $label = $widget->{_label};
       my $def   = $widget->{default};
-      my $invert_p = (($widget->{convert} || '') eq 'invert');
-
+      my $cvt   = $widget->{convert} || '';
       my $key   = "${saver}_$rsrc" if $rsrc;
 
       #### The menus don't actually have titles on X11 or Cocoa...
@@ -805,7 +810,9 @@ sub build_android(@) {
         $high_label = $high unless defined($high_label);
 
         ($low, $high) = ($high, $low)
-          if (($widget->{convert} || '') eq 'invert');
+          if ($cvt eq 'invert');
+
+        # I guess there's no way to implement convert="ratio" for Android.
 
         $settings .=
           ("<$package.SliderPreference\n" .
@@ -821,7 +828,7 @@ sub build_android(@) {
 
       } elsif ($type eq 'boolean') {
 
-        my $def = ($invert_p ? 'true' : 'false');
+        my $def = ($cvt eq 'invert' ? 'true' : 'false');
         $settings .=
           ("<CheckBoxPreference\n" .
            "  android:key=\"${key}\"\n" .
@@ -965,6 +972,7 @@ sub build_android(@) {
                        $localize0->("${saver_underscore}_saver_desc",
                                     $daydream_desc) . "\"\n" .
                   "  android:name=\".gen.Wallpaper\$$saver_class\"\n" .
+                  "  android:exported=\"true\"\n" .
                   "  android:permission=\"android.permission" .
                        ".BIND_WALLPAPER\">\n" .
                   "  <intent-filter>\n" .
@@ -1037,6 +1045,7 @@ sub build_android(@) {
   $manifest .= ("<activity android:name=\"" .
                 "$package.Activity\"\n" .
                 "  android:theme=\"\@android:style/Theme.Holo\"\n" .
+                "  android:exported=\"true\"\n" .
                 "  android:label=\"\@string/app_name\">\n" .
                 "  <intent-filter>\n" .
                 "    <action android:name=\"android.intent.action" .
@@ -1058,6 +1067,7 @@ sub build_android(@) {
   $manifest .= ("<activity android:name=\"" .
                 "$package.TVActivity\"\n" .
                 "  android:theme=\"\@android:style/Theme.Holo\"\n" .
+                "  android:exported=\"true\"\n" .
                 "  android:label=\"\@string/app_name\">\n" .
                 "  <intent-filter>\n" .
                 "    <action android:name=\"android.intent.action" .
@@ -1091,6 +1101,19 @@ sub build_android(@) {
                "  android:versionCode=\"$versb\"\n" .
                "  android:versionName=\"$vers\">\n" .
 
+               # Without this shit we get "APP_PLATFORM android-18 is higher
+               # than android:minSdkVersion 1 in AndroidManifest.xml"
+               #
+               # But with it "lintVitalRelease" complains with:
+               # "The minSdk version should not be declared in the android
+               # manifest file. You can move the version from the manifest to
+               # the defaultConfig in the build.gradle file."
+               #
+               # That crap is already in android/xscreensaver/build.gradle.
+               #
+               # "  <uses-sdk android:minSdkVersion=\"18\"" .
+               # " android:targetSdkVersion=\"30\" />\n" .
+
                "  <uses-feature android:glEsVersion=\"0x00010001\"\n" .
                "    android:required=\"true\" />\n" .
 
@@ -1213,6 +1236,9 @@ sub usage() {
 }
 
 sub main() {
+  binmode (STDOUT, ':utf8');
+  binmode (STDERR, ':utf8');
+
   my $android_p = 0;
   my @files = ();
   while ($#ARGV >= 0) {
@@ -1229,7 +1255,7 @@ sub main() {
   usage unless ($#files >= 0);
   my $failures = 0;
   foreach my $file (@files) {
-    $failures += check_config ($file);
+    $failures += check_config ($file, $android_p);
   }
 
   build_android (@files) if ($android_p);
