@@ -54,17 +54,6 @@
 
 #ifdef USE_GL /* whole file */
 
-#ifdef HAVE_XMU
-# ifndef VMS
-#  include <X11/Xmu/Drawing.h>
-#else  /* VMS */
-#  include <Xmu/Drawing.h>
-# endif /* VMS */
-#endif
-
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -295,14 +284,12 @@ reshape_gflux(ModeInfo *mi, int width, int height)
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
 
-# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
-    {
-      GLfloat h = MI_HEIGHT(mi) / (GLfloat) MI_WIDTH(mi);
-      int o = (int) current_device_rotation();
-      if (o != 0 && o != 180 && o != -180)
-        glScalef (1/h, 1/h, 1/h);
-    }
-# endif
+  {
+    GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                 ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                 : 1);
+    glScalef (s, s, s);
+  }
 }
 
 
