@@ -1,4 +1,4 @@
-/* molecule, Copyright (c) 2001-2016 Jamie Zawinski <jwz@jwz.org>
+/* molecule, Copyright (c) 2001-2016 Jamie Zawinskin <jwz@jwz.org>
  * Draws molecules, based on coordinates from PDB (Protein Data Base) files.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -27,15 +27,13 @@
 #define DEFAULTS	"*delay:	10000         \n" \
 			"*showFPS:      False         \n" \
 			"*wireframe:    False         \n" \
-	"*atomFont:   -*-helvetica-medium-r-normal-*-*-240-*-*-*-*-*-*\n" \
-	"*titleFont:  -*-helvetica-medium-r-normal-*-*-180-*-*-*-*-*-*\n" \
+			"*atomFont:     sans-serif 24\n" \
+			"*titleFont:    sans-serif 18\n" \
 			"*noLabelThreshold:    150    \n" \
 			"*wireframeThreshold:  150    \n" \
 			"*suppressRotationAnimation: True\n" \
 
 # define release_molecule 0
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
 
 #include "xlockmore.h"
 #include "colors.h"
@@ -1209,13 +1207,12 @@ reshape_molecule (ModeInfo *mi, int width, int height)
              0.0, 0.0, 0.0,
              0.0, 1.0, 0.0);
 
-# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
   {
-    int o = (int) current_device_rotation();
-    if (o != 0 && o != 180 && o != -180)
-      glScalef (1/h, 1/h, 1/h);
+    GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                 ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                 : 1);
+    glScalef (s, s, s);
   }
-# endif
 
   glClear(GL_COLOR_BUFFER_BIT);
 }
@@ -1238,15 +1235,15 @@ gl_init (ModeInfo *mi)
 static void
 startup_blurb (ModeInfo *mi)
 {
+#ifndef HAVE_ANDROID   /* Doesn't work -- causes whole scene to be black */
   molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
   const char *s = "Constructing molecules...";
-#ifndef HAVE_ANDROID   /* Doesn't work -- causes whole scene to be black */
   print_texture_label (mi->dpy, mc->title_font,
                        mi->xgwa.width, mi->xgwa.height,
                        0, s);
-#endif
   glFinish();
   glXSwapBuffers(MI_DISPLAY(mi), MI_WINDOW(mi));
+#endif
 }
 
 ENTRYPOINT Bool
@@ -1447,7 +1444,7 @@ draw_labels (ModeInfo *mi)
 
         s = 1.0 / h;			/* Scale to unit */
         s *= mc->overall_scale;		/* Scale to size of atom */
-        s *= 0.8;			/* Shrink a bit */
+        s *= 0.5;			/* Shrink a bit */
         glScalef (s, s, 1);
         glTranslatef (-w/2, -h/2, 0);
 #ifndef HAVE_ANDROID   /* Doesn't work -- causes whole scene to be black */
