@@ -37,9 +37,6 @@ Multi-coloured mode added by Jack Grahl <j.grahl@ucl.ac.uk>, Jan 2007
 
 #include "screenhack.h"
 
-#undef countof
-#define countof(x) (sizeof((x)) / sizeof(*(x)))
-
 typedef struct {
   float r, s, tx, ty;   /* Rotation, Scale, Translation X & Y */
   float ro, rt, rc;     /* Old Rotation, Rotation Target, Rotation Counter */
@@ -64,10 +61,11 @@ struct state {
   int width, widthb, height;
   int width8, height8;
   unsigned int *board;
-  XPoint pointbuf[1000];
+  XRectangle pointbuf[1000];
   int npoints;
   int xmin, xmax, ymin, ymax;
   int x, y;
+  int pscale;
 
   int delay;
 
@@ -90,6 +88,7 @@ myrandom(float up)
 }
 
 static const char *ifs_defaults [] = {
+/*  ".lowrez:             true", */
   ".background:		Black",
   "*lensnum:		3",
   "*fpsSolid:		true",
@@ -98,7 +97,7 @@ static const char *ifs_defaults [] = {
   "*colors:		200",
   "*delay:		20000",
   "*translate:		True",
-  "*scale:		True",
+  "*ifsScale:		True",
   "*rotate:		True",
   "*recurse:		False",
   "*multi:              True",
@@ -120,7 +119,7 @@ static XrmOptionDescRec ifs_options [] = {
   { "-colors",		".colors",	XrmoptionSepArg, 0 },
   { "-functions",	".lensnum", 	XrmoptionSepArg, 0 },
   { "-no-translate",	".translate",   XrmoptionNoArg, "False" },
-  { "-no-scale",	".scale",	XrmoptionNoArg, "False" },
+  { "-no-scale",	".ifsScale",	XrmoptionNoArg, "False" },
   { "-no-rotate",	".rotate",	XrmoptionNoArg, "False" },
   { "-recurse",		".recurse",	XrmoptionNoArg, "True" },
   { "-iterate",		".recurse",	XrmoptionNoArg, "False" },
@@ -136,8 +135,7 @@ static XrmOptionDescRec ifs_options [] = {
 static void
 drawpoints(struct state *st)
 {
-  XDrawPoints(st->dpy, st->backbuffer, st->gc, st->pointbuf, st->npoints,
-	      CoordModeOrigin);
+  XFillRectangles(st->dpy, st->backbuffer, st->gc, st->pointbuf, st->npoints);
   st->npoints = 0;
 }
 
@@ -162,6 +160,8 @@ sp(struct state *st, int x, int y)
 
   st->pointbuf[st->npoints].x = x;
   st->pointbuf[st->npoints].y = y;
+  st->pointbuf[st->npoints].width =
+    st->pointbuf[st->npoints].height = st->pscale;
   st->npoints++;
 
   if (st->npoints >= countof(st->pointbuf)) {
@@ -350,9 +350,10 @@ ifs_draw (Display *dpy, Window window, void *closure)
     XSetForeground(st->dpy, st->gc, st->blackColor);
     XFillRectangle(st->dpy, st->backbuffer, st->gc,
 		   xmin, ymin,
-		   xmax - xmin + 1, ymax - ymin + 1);
-    st->xmin = st->width + 1;
-    st->xmax = st->ymax = -1;
+		   xmax - xmin + 1 * st->pscale,
+                   ymax - ymin + 1 * st->pscale);
+    st->xmin = st->width  + 1;
+    st->xmax = st->ymax =  -1;
     st->ymin = st->height + 1;
   }
 
@@ -432,6 +433,12 @@ ifs_init (Display *d_arg, Window w_arg)
   XGetWindowAttributes (st->dpy, st->window, &xgwa);
   ifs_reshape(st->dpy, st->window, st, xgwa.width, xgwa.height);
 	
+  st->pscale = 1;
+  if (xgwa.width > 2560 || xgwa.height > 2560)
+    st->pscale *= 3;  /* Retina displays */
+  /* We aren't increasing the spacing between the pixels, just the size. */
+
+
   st->ncolours = get_integer_resource(st->dpy, "colors", "Colors");
   if (st->ncolours < st->lensnum)
     st->ncolours = st->lensnum;
@@ -451,7 +458,8 @@ ifs_init (Display *d_arg, Window w_arg)
   st->mode = get_integer_resource(st->dpy, "mode", "Mode");
 
   st->rotate    = get_boolean_resource(st->dpy, "rotate", "Boolean");
-  st->scale     = get_boolean_resource(st->dpy, "scale", "Boolean");
+                  /* Xft uses 'scale' */
+  st->scale     = get_boolean_resource(st->dpy, "ifsScale", "Boolean");
   st->translate = get_boolean_resource(st->dpy, "translate", "Boolean");
   st->recurse = get_boolean_resource(st->dpy, "recurse", "Boolean");
   st->multi = get_boolean_resource(st->dpy, "multi", "Boolean");
@@ -544,6 +552,7 @@ ifs_free (Display *dpy, Window window, void *closure)
   if (st->colours) free(st->colours);
   if (st->backbuffer != None && st->backbuffer != st->window)
     XFreePixmap(st->dpy, st->backbuffer);
+  XFreeGC (dpy, st->gc);
   free(st);
 }
 
