@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1991-2019 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright © 1991-2022 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -11,13 +11,11 @@
 
 /* JWXYZ Is Not Xlib.
 
-   But it's a bunch of function definitions that bear some resemblance to
-   Xlib and that do Cocoa-ish things that bear some resemblance to the
-   things that Xlib might have done.
+   This file implements Xlib in terms of Quartz / Core Graphics rendering
+   for macOS and iOS.
 
-   This is the original version of jwxyz for MacOS and iOS.
-   The version used by Android is in jwxyz-gl.c and jwxyz-common.c.
-   Both versions depend on jwxyz-cocoa.m.
+   See the comment at the top of jwxyz-common.c for an explanation of
+   the division of labor between these various modules.
  */
 
 #ifdef JWXYZ_QUARTZ // entire file
@@ -893,8 +891,8 @@ fill_rects (Display *dpy, Drawable d, GC gc,
 
     int x = rectangles[i].x;
     int y = rectangles[i].y;
-    unsigned long width = rectangles[i].width;
-    unsigned long height = rectangles[i].height;
+    long width  = rectangles[i].width;    /* Might go negative */
+    long height = rectangles[i].height;
 
     if (fast_fill_p) {
       long   // negative_int > unsigned_int == 1
@@ -1517,8 +1515,9 @@ jwxyz_png_to_ximage (Display *dpy, Visual *visual,
     [img release];
     return 0;
   }
-  int width   = [img size].width;
-  int height  = [img size].height;
+  // images/earth.png is 2048x1024 but img.size is 491x245?
+  int width   = [bm pixelsWide];
+  int height  = [bm pixelsHigh];
   size_t ibpp = [bm bitsPerPixel];
   size_t ibpl = [bm bytesPerRow];
   const unsigned char *data = [bm bitmapData];
@@ -1651,7 +1650,7 @@ copy_pixmap (Display *dpy, Pixmap p)
 
 
 // Returns the verbose Unicode name of this character, like "agrave" or
-// "daggerdouble".  Used by fontglide debugMetrics.
+// "daggerdouble".  Used by Unicrud, and by Fontglide with debugMetrics.
 //
 char *
 jwxyz_unicode_character_name (Display *dpy, Font fid, unsigned long uc)
